import pandas as pd
import numpy as np


def read_cary_oxygen(filename):
    """Import oxygen data from Agilent Cary spectrophotometer report.

    Parameters
    ----------
    filename : str
        The file name (and path) for the .csv report.

    Returns
    -------
    pd.DataFrame
        The data from the file parsed into a pandas DataFrame.
    """

    with open(filename, "r") as f:
        lines = f.read().splitlines()

    data_line = False
    oxy = {"name": [], "repeat": []}
    reps = ["r{}".format(r) for r in range(1, 6)]
    oxy.update({rep: [] for rep in reps})
    repeat = (
        0  # increments to 1 for any repeated measurements at end of first run through
    )
    for line in lines:
        # Parse date and time of analysis
        if line.startswith("Collection time"):
            mm, dd, yyyy = line.split()[2].split("/")
            HH, MM, SS = line.split()[3].split(":")
            if line.split()[4] == "PM" and HH != "12":
                HH = int(HH) + 12
            datetime_analysis = np.datetime64(
                "{:04.0f}-{:02.0f}-{:02.0f}T{:02.0f}:{:02.0f}:{:02.0f}".format(
                    int(yyyy), int(mm), int(dd), int(HH), int(MM), int(SS)
                )
            )

        # Get zero value
        if line.startswith("Zero") and not line == "Zero Report":
            zero = float(line.split()[1])

        # If this line is seen, we've reached the end of the data section:
        if line == "Results Flags Legend":
            data_line = False

        # Parse the data section here
        if data_line:
            line_split = line.split()
            # The "+ repeat" in this section deals with the fact that there is an extra
            # column containing "R" in the tables for repeated measurements
            assert len(line_split) in np.array([1, 2, 4]) + repeat
            if len(line_split) == 2 + repeat:  # first measurement for a new sample
                oxy["name"].append(line_split[0])
                oxy["repeat"].append(repeat > 0)
                oxy["r1"].append(float(line_split[1 + repeat]))
                r = 2
            elif len(line_split) == 1 + repeat:  # intermediate measurements
                oxy["r{}".format(r)].append(float(line_split[0 + repeat]))
                r += 1
            elif len(line_split) == 4 + repeat:  # final measurement and statistics
                oxy["r{}".format(r)].append(float(line_split[3 + repeat]))

        # If this line is seen, we're about to start the data section:
        if line == "Sample	F	Mean	SD	%RSD	Readings":
            data_line = True

        if line.startswith("R = Repeat reading"):
            repeat = 1

    # Convert to DataFrame and calculate statistics and other things
    oxy = pd.DataFrame(oxy)
    oxy["absorbance_raw"] = oxy[reps].mean(axis=1)
    oxy["absorbance_std"] = oxy[reps].std(axis=1)
    oxy["mq"] = oxy.name.str.startswith("MQ")
    oxy["zero"] = zero

    return oxy



import numpy as np
from . import oxygen


def pH_NIOZ(
    absorbance_578nm, absorbance_434nm, absorbance_730nm, temperature=25, salinity=35
):
    """Calculate pH from NIOZ spectrophotometer absorbances.

    Parameters
    ----------
    absorbance_578nm : array_like
        Absorbance at 578 nm.
    absorbance_434nm : array_like
        Absorbance at 434 nm.
    absorbance_730nm : array_like
        Absorbance at 730 nm.
    temperature : array_like, optional
        Temperature in °C, by default 25.
    salinity : array_like, optional
        Practical salinity, by default 35.

    Returns
    -------
    array_like
        pH on the total scale.
    """
    WL1 = absorbance_578nm
    WL2 = absorbance_434nm
    WL3 = absorbance_730nm
    pH_total = (
        np.log10(
            (((WL1 - WL3) / (WL2 - WL3) - 0.00815 * WL1) - 0.00691)
            / (2.222 - ((WL1 - WL3) / (WL2 - WL3) - 0.00815 * WL1) * 0.1331)
        )
        + 1245.69 / (temperature + 273.15)
        + 3.8275
        + 0.00211 * (35 - salinity)
    )
    return pH_total


def pH_DSC07(
    absorbance_578nm,
    absorbance_434nm,
    absorbance_730nm,
    temperature=25,
    salinity=35,
    dye_intercept=0,
    dye_slope=1,
):
    """Calculate pH following SOP 6b, including dye addition correction if known.

    Parameters
    ----------
    absorbance_578nm : array_like
        Absorbance at 578 nm.
    absorbance_434nm : array_like
        Absorbance at 434 nm.
    absorbance_730nm : array_like
        Absorbance at 730 nm.
    temperature : array_like, optional
        Temperature in °C, by default 25.
    salinity : array_like, optional
        Practical salinity, by default 35.
    dye_intercept : array_like, optional
        Intercept of the dye correction (SOP 6b eq. 9), by default 0.
    dye_slope : array_like, optional
        Slope of the dye correction (SOP 6b eq. 9), by default 1.

    Returns
    -------
    array_like
        pH on the total scale.
    """
    WL1 = absorbance_578nm
    WL2 = absorbance_434nm
    WL3 = absorbance_730nm
    ratio_raw = (WL1 - WL3) / (WL2 - WL3)
    ratio = ratio_raw - (dye_intercept + dye_slope * ratio_raw)
    pH_total = (
        np.log10((ratio - 0.00691) / (2.222 - ratio * 0.1331))
        + 1245.69 / (temperature + 273.15)
        + 3.8275
        + 0.00211 * (35 - salinity)
    )
    return pH_total
